'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _codemirror = require('codemirror');

var _codemirror2 = _interopRequireDefault(_codemirror);

require('codemirror/mode/javascript/javascript');

require('codemirror/addon/lint/lint');

require('codemirror/addon/lint/javascript-lint');

require('codemirror/addon/runmode/runmode');

var codeExamples = {
    init: init,
    all: initialiseExamples
};
var coderunner = undefined;
var initSuccess = undefined;
var initFailed = undefined;
var codeRunnerInitialized = new Promise(function (resolve, reject) {
    initSuccess = resolve;
    initFailed = reject;
});

function init() {
    var exampleRunner = arguments[0] === undefined ? './examplerunner.js' : arguments[0];
    var options = arguments[1] === undefined ? {} : arguments[1];

    addStylesToHead();

    //Start the worker that runs the example
    coderunner = new Worker(exampleRunner);
    coderunner.addEventListener('message', initListener);

    function initListener(event) {
        if (event.data === 'init-complete') {
            initSuccess();
        }
        coderunner.removeEventListener('message', initListener);
    }

    //Initialise the worker with the settings for api communication
    coderunner.postMessage({
        type: 'init',
        options: options
    });

    return codeExamples;
}

function initialiseExamples() {
    if (hasWebWorkerSupport() && hasFetchSupport()) {
        runnableCodeExamples();
        staticCodeExamples();
    } else {
        //TODO: Show a browser message thing that will notify user of no examples?
        console.info && console.info('No web worker and/or fetch support in this browser therefore we will render static examples only');
        staticCodeExamples(true);
    }
}

function addStylesToHead() {
    var stylesForCodeExamples = '\n    .d2-runnable-code-example-run-button {\n            width: 100%;\n            -webkit-appearance: none;\n            -moz-appearance: none;\n            background: none;\n            border: none;\n            background: rgb(33,150,243);\n            color: white;\n            font-size: 1rem;\n            padding: 6px;\n        }\n\n        .d2-static-code-example {\n            padding: 1rem 2rem;\n            border-left: .5rem solid;\n            border-color: rgb(33,150,243);\n        }\n\n        .CodeMirror {\n            height: auto;\n        }\n    ';

    var styleElement = document.createElement('style');
    styleElement.setAttribute('type', 'text/css');
    styleElement.textContent = stylesForCodeExamples;

    document.querySelector('head').appendChild(styleElement);
}

function runnableCodeExamples() {
    var runnableCodeExampleElements = getRunnableCodeExamples();

    runnableCodeExampleElements.forEach(function (element) {
        var codeMirrorElement = document.createElement('div');
        var runButton = getRunButton();
        var options = getCodeMirrorOptions();
        var codeMirror = undefined;

        //Navigate to the node above the <pre>
        element.parentNode.parentNode.insertBefore(codeMirrorElement, element.parentNode);

        //Set the code as the value for codemirror
        options.value = element.textContent.trim() + '\n';

        //TODO: Code mirror needs to be run on the element that is already in the DOM?
        codeMirror = (0, _codemirror2['default'])(codeMirrorElement, options);

        if (window.JSHINT) {
            (function () {
                var waiting = undefined;
                codeMirror.on('change', function () {
                    clearTimeout(waiting);
                    waiting = setTimeout(updateHints(codeMirror), 500);
                });

                setTimeout(updateHints(codeMirror), 100);
            })();
        }

        runButton.data = {
            codeMirror: codeMirror
        };

        codeMirrorElement.appendChild(runButton);

        element.style.display = 'none';
    });
}

function updateHints(editor) {
    return function () {
        editor.operation(function () {
            JSHINT(editor.getValue(), { esnext: true });
            for (var i = 0; i < JSHINT.errors.length; ++i) {
                var err = JSHINT.errors[i];
                if (!err) continue;

                //TODO: Do something to display the hints if JSHINT is available
                //console.info(`${err.line}: ${err.reason}`);
            }
        });
    };
}

function staticCodeExamples(includeRunnableAsStatic) {
    var exampleElements = getCodeExamples();

    if (includeRunnableAsStatic) {
        exampleElements = exampleElements.concat(getRunnableCodeExamples());
    }

    exampleElements.forEach(function (element) {
        _codemirror2['default'].runMode(element.textContent + '\n', getCodeMirrorOptions(), element);

        element.parentNode.classList.add('cm-s-monokai', 'CodeMirror', 'd2-static-code-example');
    });
}

function hasWebWorkerSupport() {
    return !!window.Worker;
}

function hasFetchSupport() {
    return !!window.fetch;
}

function getRunnableCodeExamples() {
    return getNonEmptyCodeElementsWithClass('d2-runnable-code-example');
}

function getCodeExamples() {
    return getNonEmptyCodeElementsWithClass('d2-code-example');
}

function getNonEmptyCodeElementsWithClass(className) {
    var runnableCodeElements = Array.from(document.querySelectorAll('code.' + className));

    return runnableCodeElements.filter(function (element) {
        return element.textContent.trim();
    });
}

function getCodeMirrorOptions() {
    return {
        name: 'javascript',
        lineNumbers: true,
        theme: 'monokai'
    };
}

function getRunButton() {
    var runButtonElement = document.createElement('button');
    runButtonElement.textContent = 'Initialising D2..';
    runButtonElement.disabled = true;
    runButtonElement.classList.add('d2-runnable-code-example-run-button');

    codeRunnerInitialized.then(function () {
        runButtonElement.textContent = 'Run!';
        runButtonElement.disabled = false;
    });

    runButtonElement.addEventListener('click', function (event) {
        if (this.data && this.data.codeMirror) {
            var element = this;
            var oldButtonText = this.textContent;
            var code = this.data.codeMirror.getValue();

            if (!code) {
                return;
            }

            //TODO: Change this for an addEventListener that de-registers
            coderunner.onmessage = function () {
                event.target.textContent = 'Run!';
            };

            this.textContent = 'Running...';
            coderunner.postMessage(code);
        }
    });

    return runButtonElement;
}

exports['default'] = codeExamples;
module.exports = exports['default'];