'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _d2LibUtils = require('d2/lib/utils');

var _d2LibCheck = require('d2/lib/check');

var _d2LoggerLogger = require('d2/logger/Logger');

var _d2LoggerLogger2 = _interopRequireDefault(_d2LoggerLogger);

var _d2ModelModels = require('d2/model/models');

var _d2ModelModels2 = _interopRequireDefault(_d2ModelModels);

var _d2ApiApi = require('d2/api/Api');

var _d2ApiApi2 = _interopRequireDefault(_d2ApiApi);

var _d2SystemSystem = require('d2/system/System');

var _d2SystemSystem2 = _interopRequireDefault(_d2SystemSystem);

/**
 * @function d2Init
 *
 * @param {Object} config Configuration object that will be used to configure to define D2 Setting.
 * See the description for more information on the available settings.
 * @returns {Promise} A promise that resolves with the intialized d2 object. Which is an object that exposes `model`, `models` and `Api`
 *
 * @description
 * Init function that used to initialise D2. This will load the schemas from the DHIS2 api and configure your D2 instance.
 *
 * The `options` object that can be passed into D2 can have the following properties:
 *
 * baseUrl: Set this when the url is something different then `/api`. If you are running your dhis instance in a subdirectory of the actual domain
 * for example http://localhost/dhis/ you should set the base url to `/dhis/api`
 *
 * ```js
 * import d2Init from 'd2';
 *
 * d2Init({baseUrl: '/dhis/api'})
 *   .then((d2) => {
 *     console.log(d2.model.dataElement.list());
 *   });
 * ```
 */
function d2Init(config) {
    var logger = _d2LoggerLogger2['default'].getLogger();

    var d2 = {
        models: undefined,
        model: _d2ModelModels2['default'],
        Api: _d2ApiApi2['default'],
        system: _d2SystemSystem2['default'].getSystem()
    };

    var api = _d2ApiApi2['default'].getApi();

    if (config && (0, _d2LibCheck.checkType)(config, 'object', 'Config parameter')) {
        processConfig(api, config);
    }

    _d2ModelModels2['default'].ModelDefinition.prototype.api = api;

    d2.models = new _d2ModelModels2['default'].ModelDefinitions();

    return api.get('schemas').then((0, _d2LibUtils.pick)('schemas')).then(function (schemas) {
        schemas.forEach(function (schema) {
            d2.models.add(_d2ModelModels2['default'].ModelDefinition.createFromSchema(schema));
        });

        return d2;
    })['catch'](function (error) {
        logger.error('Unable to get schemas from the api', error);

        return Promise.reject(error);
    });
}

function processConfig(api, config) {
    if ((0, _d2LibCheck.isString)(config.baseUrl)) {
        api.setBaseUrl(config.baseUrl);
    } else {
        api.setBaseUrl('/api');
    }
}

if (typeof window !== 'undefined') {
    window.d2 = d2Init;
}

exports['default'] = d2Init;
module.exports = exports['default'];