'use strict';
Object.defineProperty(exports, '__esModule', {
    value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _d2ModelModelValidation = require('d2/model/ModelValidation');

var _d2ModelModelValidation2 = _interopRequireDefault(_d2ModelModelValidation);

var modelValidator = _d2ModelModelValidation2['default'].getModelValidation();

var DIRTY_PROPERTY_LIST = Symbol('List to keep track of dirty properties');

exports.DIRTY_PROPERTY_LIST = DIRTY_PROPERTY_LIST;
/**
 * @class ModelBase
 */

var ModelBase = (function () {
    function ModelBase() {
        _classCallCheck(this, ModelBase);
    }

    _createClass(ModelBase, [{
        key: 'save',

        /**
         * @method save
         *
         * @returns {Promise} Returns a promise that resolves when the model has been saved
         * or rejects with the result from the `validate()` call.
         *
         * @description
         * Checks if the model is dirty. When the model is dirty it will check if the values of the model are valid by calling
         * `validate`. If this is correct it will attempt to save the [Model](#/model/Model) to the api.
         *
         * ```js
         * myModel.save()
         *   .then((message) => console.log(message));
         * ```
         */
        value: function save() {
            var _this = this;

            if (!this.dirty) {
                return Promise.reject('No changes to be saved');
            }

            return this.validate().then(function (validationState) {
                if (!validationState.status) {
                    return Promise.reject(validationState);
                }

                return _this.modelDefinition.save(_this).then(function (result) {
                    _this.dirty = false;
                    _this[DIRTY_PROPERTY_LIST].clear();
                    return result;
                });
            });
        }
    }, {
        key: 'validate',

        /**
         * @method validate
         *
         * @returns {Promise} Promise that resolves with an object with a status property that represents if the model
         * is valid or not the fields array will return the names of the fields that are invalid.
         *
         * @description
         * This will run the validations on the properties which have validations set. Normally these validations are defined
         * through the DHIS2 schema. It will check min/max for strings/numbers etc. Additionally it will
         * run model validations against the schema.
         *
         * ```js
         * myModel.validate()
         *  .then(myModelStatus => {
         *    if (myModelStatus.status === false) {
         *      myModelStatus.fields.forEach((fieldName) => console.log(fieldName));
         *    }
         * });
         * ```
         */
        value: function validate() {
            var _this2 = this;

            return new Promise(function (resolve, reject) {
                var modelValidationStatus = true;
                var validationMessages = [];
                var validationState = undefined;

                //Run local validation on the models data values
                validationMessages = validationMessages.concat(localValidation(_this2.modelDefinition.modelValidations, _this2.dataValues));

                //Run async validation against the api
                asyncRemoteValidation(_this2).then(function (remoteMessages) {
                    validationMessages = validationMessages.concat(remoteMessages);

                    validationState = {
                        status: modelValidationStatus,
                        fields: validationMessages.map(function (validationMessage) {
                            return validationMessage.property;
                        }).reduce(unique, []),
                        messages: validationMessages
                    };
                    resolve(validationState);
                })['catch'](function (message) {
                    return reject(message);
                });

                function unique(current, property) {
                    if (property && current.indexOf(property) === -1) {
                        current.push(property);
                    }
                    return current;
                }

                function localValidation(modelValidations, dataValues) {
                    var validationMessagesLocal = [];

                    Object.keys(modelValidations).forEach(function (propertyName) {
                        var validationStatus = modelValidator.validate(modelValidations[propertyName], dataValues[propertyName]);
                        if (!validationStatus.status) {
                            validationStatus.messages.forEach(function (message) {
                                message.property = propertyName;
                            });
                        }
                        modelValidationStatus = modelValidationStatus && validationStatus.status;
                        validationMessagesLocal = validationMessagesLocal.concat(validationStatus.messages || []);
                    });

                    return validationMessagesLocal;
                }

                function asyncRemoteValidation(model) {
                    return modelValidator.validateAgainstSchema(model);
                }
            });
        }
    }, {
        key: 'getDirtyPropertyNames',
        value: function getDirtyPropertyNames() {
            return Array.from(this[DIRTY_PROPERTY_LIST].values());
        }
    }]);

    return ModelBase;
})();

exports['default'] = new ModelBase();